package de.fhw.swprojekt.knauf.java3d;

import java.awt.Color;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingBox;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.Node;
import javax.media.j3d.PickBounds;
import javax.media.j3d.SceneGraphPath;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.View;
import javax.swing.JFrame;
import javax.vecmath.Color3f;
import javax.vecmath.Point2d;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

import com.sun.j3d.utils.geometry.Primitive;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.SimpleUniverse;

/**Fenster fr eine Anwendung mit simpler Userinteraktion: 
 * a) Bereich fr eine Sphere definieren (Linksklick gibt linkes Zentrum an, dann 
 *    Maus ziehen fr Durchmesser).
 * b) Rechtsklick lscht alle Spheres unter der Maus.
 * 
 * @author Wolfgang Knauf
 * 
 */
public class Java3dInteractionFrame extends JFrame
{
  /**SerialVersionUID (damit Eclipse schweigt)
   * 
   */
  private static final long serialVersionUID = 1L;

  /**Das eigentliche Zeichenpanel
   * 
   */
  private Canvas3D canvas3D = null;
  
  /**Das Universum, dass die 3D-Daten enthlt.
   * 
   */
  private SimpleUniverse universe = null;
  
  /**Die BranchGroup enthlt alle Elemente.
   * 
   */
  private BranchGroup branchgroup;
  
  /**Konstruktor des Fensters. Hier wird das Universum initialisiert.
   * 
   * @param title Fenstertitel
   */
  public Java3dInteractionFrame(String title)
  {
    super(title);

    // AnzeigeInfos holen (Farbtiefe, usw...
    GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();

    // ein neues Canvas mit den AnzeigeInfos erzeugen und dem Frame hinzufgen
    this.canvas3D = new Canvas3D (config);
    add(canvas3D);

    // ein neues Universum im Canvas erzeugen und eine Betrachtungsebene erzeugen
    this.universe = new SimpleUniverse(canvas3D);
    this.universe.getViewingPlatform().setNominalViewingTransform();
    
    //neue Branchgruppe
    this.branchgroup = new BranchGroup();
    //"AllowDetach" muss gesetzt sein, damit man sie aus dem Universe entfernen kann.
    //(im aktuellen Beispielcode nicht mehr ntig)
    //this.branchgroup.setCapability(BranchGroup.ALLOW_DETACH);
    //Fr Lschen von Children:
    this.branchgroup.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    //Fr Zufgen von weiteren Children:
    this.branchgroup.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);

    // ... an das Universum hngen
    universe.addBranchGraph(this.branchgroup);
   
    //Skalierung verhindern:
    canvas3D.getView().setWindowEyepointPolicy(View.RELATIVE_TO_SCREEN); 
    canvas3D.getView().setWindowMovementPolicy(View.VIRTUAL_WORLD);
    canvas3D.getView().setWindowResizePolicy(View.VIRTUAL_WORLD);
    
    //Antialiasing einschalten:
    this.canvas3D.getView().setSceneAntialiasingEnable(true);
  
    //Mausevent-Handler:
    //Click-listener zufgen (als anonymer MouseListener bzw. dessen implementierende Dummy-Klasse "MouseAdapter"):
    this.canvas3D.addMouseListener( new MouseAdapter ()
    {
      /**Startposition (x-Koordinate) der Mausmarkierung in Welt-Koordinaten (also Werte zwischen -1.0 und +1.0!)
       * Wird beim Mausklick gesetzt. Beim Maus-Loslassen wird aus der Breite des Zeichenbereichs der 
       * Radius des Kreises ermittelt.
       * 
       */
      private double dblStartX = 0.0;
      
      /**Y-Koordinate der Mausmarkierung in Welt-Koordinaten
       */
      private double dblStartY = 0.0;
      
      /**Die Maus wird gedrckt: Startpunkt des Rechtecks speichern.
       * 
       */
      @Override
      public void mousePressed(MouseEvent e)
      {
        System.out.println("Click an " + e.getX() + "/" + e.getY());
        
        Point2d pointMaus = convertMousePositionToWorld(e.getX(), e.getY());
        this.dblStartX = pointMaus.x;
        this.dblStartY = pointMaus.y;
        
        System.out.println ("x/y: " + this.dblStartX + "/" + this.dblStartY);
        
        //Bei rechter Mausteste: gewhlte Items holen und lschen:
        if (e.getButton() == MouseEvent.BUTTON3)
        {
          //Fr Prfen der aktuellen Auswahl wird ein "PickBounds"-Objekt mit einer BoundingBox, die 
          //genau den Punkt des Mausklicks umfasst, erzeugt.
          //z-Koordinate des Punktes ist "0":
          Point3d pickPoint = new Point3d (this.dblStartX, this.dblStartY, 0.0);
          PickBounds pickBounds = new PickBounds ( new BoundingBox (pickPoint, pickPoint));
          SceneGraphPath[] pickedItems = branchgroup.pickAll(pickBounds);
          if (pickedItems != null)
          {
            System.out.println ("Picked Items: " + pickedItems.length);
            for (int intIndexSceneGraph = 0; intIndexSceneGraph < pickedItems.length; intIndexSceneGraph++)
            {
              System.out.println ("SceneGraphPath " + intIndexSceneGraph + ": " + pickedItems[intIndexSceneGraph].nodeCount());
              //ber die Nodes laufen:
              for (int intIndexNode = 0; intIndexNode < pickedItems[intIndexSceneGraph].nodeCount(); intIndexNode++)
              {
                //Jeder der Nodes sollte eine Sphere sein:
                Node nodeAktuell = pickedItems[intIndexSceneGraph].getNode(intIndexNode);
                System.out.println (nodeAktuell);
                
                //Parent des Knotens ist die TransformGroup:
                TransformGroup transformGroupRemove = (TransformGroup) nodeAktuell.getParent();
                //Deren Parent ist wiederum eine BranchGroup (das wei ich, weil der Code im "drawSphere"
                //dies so erzeugt:
                BranchGroup branchGroupRemove = (BranchGroup) transformGroupRemove.getParent();
                
                //Und diese BranchGroup knnen wir aus der Toplevel-Branchgroup des Universums kicken:
                branchgroup.removeChild (branchGroupRemove);
              }
            }
          }
          else
          {
            System.out.println ("Nix gewhlt!");
          }
        } //Ende if (rechte Maustaste)
      }

      /**Maus wird losgelassen: Rechteck fertigzeichnen.
       * @param e Mauskoordinaten etc.
       */
      @Override
      public void mouseReleased(MouseEvent e)
      {
        System.out.println("mouseReleased an " + e.getX() + "/" + e.getY());
        //Nur bei linker Maustaste etwas tun:
        if (e.getButton() == MouseEvent.BUTTON1)
        {
          //Jetzt knnen wir den Durchmesser der Sphere ermitteln:
          
          Point2d pointMaus = convertMousePositionToWorld(e.getX(), e.getY());
          //Durchmesser ist: aktuelles x - Start-X.
          double dblWidth = pointMaus.x - this.dblStartX;
          
          System.out.println ("Von " + this.dblStartX + "/" + this.dblStartY + " => " + dblWidth);
          
          //Zeichnen:
          drawSphere(this.dblStartX, this.dblStartY, dblWidth);
        }
      }
      
    } );
  }

  /**Main-Methode, zeigt das Fenster fr Java3D mit Interaktion an.
   * 
   * 
   * @param args Kommandozeile, ungenutzt.
   */
  public static void main(String args[])
  {
    Java3dInteractionFrame java3dFrame = new Java3dInteractionFrame("Interaktion");
    java3dFrame.setSize(600, 400);
    java3dFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    java3dFrame.setExtendedState(Frame.MAXIMIZED_BOTH);
    java3dFrame.setVisible(true);
  }
  
  /**Sphere an Stelle x/y (Mittelpunkt!) mit bergebener Breite zeichenen.
   *  
   */
  private void drawSphere (double dX, double dY, double dWidth)
  {
    //Erscheinungsbild des Kreises: gelb.
    Appearance appearanceSphere = new Appearance();
    ColoringAttributes coloringAttributes = new ColoringAttributes();
    coloringAttributes.setColor(new Color3f(Color.yellow));
    appearanceSphere.setColoringAttributes(coloringAttributes);
    
    
    //Die errechnete Breite durch 2 teilen, da wir den Radius angeben mssen.
    float radius = (float) (dWidth / 2.0);
    //Insgesamt soll der Rand aus 100 Segmenten bestehen (je mehr, desto runder sieht sie aus).
    //Der Parameter "primitiveFlags" scheint per Default auf "GenerateNormals" (=1) zu stehen,
    //also bergebe ich das hier.
    Sphere sphereNew = new Sphere (radius, Primitive.GENERATE_NORMALS, 100, appearanceSphere);
    
    Transform3D transformSphere = new Transform3D();
    //x/y verschieben:
    //Bei x-Verschiebung den Radius aufrechnen! Beim y nicht. Z-Koordinate ist natrlich 0.
    transformSphere.setTranslation(new Vector3d (dX + radius,dY,0));
    
    
    //neue Transformgruppe
    TransformGroup transformGroupSphere = new TransformGroup(transformSphere);
   
    //colorcube an Transformgruppe hngen
    transformGroupSphere.addChild(sphereNew);
    
    //JEDE Sphere muss in eine eigene BranchGroup, damit man lschen kann!
    BranchGroup branchGroupSphereAktuell = new BranchGroup();
    //Hier muss "DETACH" erlaubt sein:
    branchGroupSphereAktuell.setCapability(BranchGroup.ALLOW_DETACH);
    
    branchGroupSphereAktuell.addChild(transformGroupSphere);
    
    //Und diese komme in Gesamt-Branchgroup:
    this.branchgroup.addChild(branchGroupSphereAktuell);
  }
  
  /**Konvertieren eine x/y-Mausposition in x/y-"Weltkoordinaten". Die z-Achse spielt hier keine Rolle.
   * 
   * @param iX X-Koordinaten eines Punkts im aktuellen Fenster in Pixeln
   * @param iY Y-Koordinaten eines Punkts im aktuellen Fenster in Pixeln
   * @return Neuer Punkt mit x/y-Weltkoordinaten
   */
  private Point2d convertMousePositionToWorld (int iX, int iY)
  {
    //Quelle:
    //http://forums.java.net/jive/thread.jspa?messageID=237589
    
    Point3d eye_pos = new Point3d();
    Point3d mousePosn = new Point3d();

    //get the eye point and mouse click point
    this.canvas3D.getCenterEyeInImagePlate(eye_pos);
    this.canvas3D.getPixelLocationInImagePlate(iX, iY, mousePosn);

    //Transform from ImagePlate coordinates to Vworld coordinates
    Transform3D motion = new Transform3D();
    this.canvas3D.getImagePlateToVworld(motion);

    motion.transform(eye_pos);
    motion.transform(mousePosn);

    //calculate the intersection point on Z=0
    double dblX = (-eye_pos.z / (mousePosn.z - eye_pos.z))
    * (mousePosn.x - eye_pos.x) + eye_pos.x;
    double dblY = (-eye_pos.z / (mousePosn.z - eye_pos.z))
    * (mousePosn.y - eye_pos.y) + eye_pos.y;
    
    return new Point2d (dblX, dblY);
  }
}
