package de.fhw.swprojekt.knauf.java3d;

import java.awt.Color;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;

import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.DirectionalLight;
import javax.media.j3d.Material;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.swing.JFrame;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.Cone;
import com.sun.j3d.utils.universe.SimpleUniverse;

/**Beispiel fr simple Beleuchtung.
 * 
 * @author Wolfgang Knauf
 *
 */
public class Java3dLightFrame extends JFrame
{
  /**SerialVersionUID (damit Eclipse schweigt)
   * 
   */
  private static final long serialVersionUID = 1L;

  /**Konstruktor des Fensters, hier wird auerdem die Objekthierarchie erzeugt.
   * 
   * @param title Fenstertitel
   */
  public Java3dLightFrame(String title)
  {
    super(title);

    // AnzeigeInfos holen (Farbtiefe, usw...
    GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();

    // ein neues Canvas mit den AnzeigeInfos erzeugen und dem Frame hinzufgen
    Canvas3D canvas3D = new Canvas3D (config);
    add(canvas3D);

    // ein neues Universum im Canvas erzeugen und eine Betrachtungsebene erzeugen
    SimpleUniverse universe = new SimpleUniverse(canvas3D);
    
    //Weitere Initialisierung des Canvas:
    //Antialiasing einschalten:
    canvas3D.getView().setSceneAntialiasingEnable(true);
    
    //Hier wird das Default-Verhalten verwendet: Szenerie ist vollstndig sichtbar
    //und skaliert sich mit dem Fenster.
    universe.getViewingPlatform().setNominalViewingTransform();
    
    
    //=====================================================
    //Zuerst das Material der Boxes erzeugen:
    Appearance appearanceBox = new Appearance();

    //Die Box soll beleuchtbar sein.
    Material materialBox = new Material();
    //Bei diffuser Beleuchtung soll das Material dunkelgrn sein.
    materialBox.setDiffuseColor(new Color3f(0.0f, 0.5f, 0.0f));
    //Die direkt angestrahlte Stelle soll heller wirken:
    materialBox.setSpecularColor(new Color3f(0.0f, 1.0f, 0.0f));

    appearanceBox.setMaterial(materialBox);

    //=====================================================
    //Transform fr Box 1 (man schaut direkt drauf) :

    Box box1 = new Box (0.2f, 0.1f, 0.2f, appearanceBox);

    //Rechtsverschiebung:
    Transform3D transform3dBox1 = new Transform3D();
    transform3dBox1.setTranslation(new Vector3d (0.6f,0.0,0));
    
    // neue Transformgruppe
    TransformGroup transformGroupBox1 = new TransformGroup(transform3dBox1);

    //Box an Transformgruppe hngen
    transformGroupBox1.addChild(box1);
    
    
    //=====================================================
    //Transform fr die Box 2 (leicht gedreht):
    Box box2 = new Box (0.2f, 0.1f, 0.2f, appearanceBox);

    //Erst Rotation um die y-Achse, dann Rechtsverschiebung:
    Transform3D transform3dBox2 = new Transform3D();
    transform3dBox2.rotY(Math.PI / 4);
    transform3dBox2.setTranslation(new Vector3d (0.6f,0.3f,0));
    
    // neue Transformgruppe
    TransformGroup transformGroupBox2 = new TransformGroup(transform3dBox2);

    //Box an Transformgruppe hngen
    transformGroupBox2.addChild(box2);
    //=====================================================
    //Kegel!
    
    Appearance appearanceCone = new Appearance();
    
    //Der Kegel soll beleuchtbar sein.
    Material materialCone = new Material();
    //Bei diffuser Beleuchtung soll er leicht rot wirken.
    materialCone.setDiffuseColor(new Color3f(0.4f, 0.0f, 0.0f));
    //Die direkt angestrahlte Stelle soll heller wirken:
    materialCone.setSpecularColor(new Color3f(1.0f, 1.0f, 1.0f));
    //Die Specular Color soll stark durchkommen:
    materialCone.setShininess(1.0f);
    appearanceCone.setMaterial(materialCone);
    
    //Radius/Hhe:
    Cone cone = new Cone (0.2f, 0.8f, appearanceCone);

    Transform3D transform3dCone = new Transform3D();
    transform3dCone.setTranslation(new Vector3d (-0.2f,0, 0));
    //Um 45 Grad an der x-Achse rotieren => Spitze geht auf User zu.
    transform3dCone.rotX( Math.PI / 4);
    
    // neue Transformgruppe
    TransformGroup transformGroupCone = new TransformGroup(transform3dCone);

    //Cone an Transformgruppe hngen
    transformGroupCone.addChild(cone);
    
    
    //================================================
    //LICHT an:
    //Das gesamte Universum wird angestrahlt (Kugel am Koordinatenursprung mit Radius 1000)
    BoundingSphere worldBounds = new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 1000.0);

    //Weies Licht bauen und einschalten:
    DirectionalLight light = new DirectionalLight();
    light.setInfluencingBounds(worldBounds);
    light.setColor (new Color3f (Color.white));
    light.setEnable(true);
    
    //================================================
    //Alles zusammenbauen:
   
    
    //neue Branchgruppe
    BranchGroup branchgroup = new BranchGroup();
    
    //Transformgruppen an Branchgruppe hngen
    branchgroup.addChild(transformGroupBox1);
    branchgroup.addChild(transformGroupBox2);
    branchgroup.addChild(transformGroupCone);
    //Auch das Licht kommt dazu:
    branchgroup.addChild(light);
    
    // ... an das Universum hngen
    universe.addBranchGraph(branchgroup);
  }

  /**Main-Methode, zeigt das Fenster fr Java3D mit Beleuchtung an.
   * 
   * @param args Kommandozeile, ungenutzt.
   */
  public static void main(String args[])
  {
    Java3dLightFrame java3dFrame = new Java3dLightFrame("Java3D mit Licht");
    java3dFrame.setSize(600, 400);
    java3dFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    java3dFrame.setExtendedState(Frame.MAXIMIZED_BOTH);
    
    java3dFrame.setVisible(true);
  }
}
