package de.fhw.swprojekt.knauf.java3d;

import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.LineArray;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.View;
import javax.swing.JFrame;
import javax.vecmath.Color3f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3d;

import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.Cone;
import com.sun.j3d.utils.geometry.Text2D;
import com.sun.j3d.utils.universe.SimpleUniverse;

/**Testframe fr einige grundlegende Tests von Java3D
 * 
 * @author Wolfgang Knauf
 * 
 */
public class Java3dTestFrame extends JFrame
{
  /**SerialVersionUID (damit Eclipse schweigt)
   * 
   */
  private static final long serialVersionUID = 1L;

  /**Konstruktor des Fensters, hier wird auerdem die Objekthierarchie erzeugt.
   * 
   * @param title Fenstertitel
   */
  public Java3dTestFrame(String title)
  {
    super(title);

    // AnzeigeInfos holen (Farbtiefe, usw...
    GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();

    // ein neues Canvas mit den AnzeigeInfos erzeugen und dem Frame hinzufgen
    Canvas3D canvas3D = new Canvas3D (config);
    add(canvas3D);

    // ein neues Universum im Canvas erzeugen und eine Betrachtungsebene erzeugen
    SimpleUniverse universe = new SimpleUniverse(canvas3D);
    
    //Weitere Initialisierung des Canvas:
    //Antialiasing einschalten:
    canvas3D.getView().setSceneAntialiasingEnable(true);
    
    //Variante 1: Default-Verhalten mit Skalierung
    universe.getViewingPlatform().setNominalViewingTransform();
    
    //Variante 2: verkleinerter Sichtbereich:
    //canvas3D.getView().setFieldOfView(0.1);
    //universe.getViewingPlatform().setNominalViewingTransform();
    //Hierzu MUSS die WindowEyepointPolicy auf "RELATIVE_TO_SCREEN" oder "RELATIVE_TO_WINDOW" gendert werden!
    
    //Optional: nicht-skalieren des Sichtbereichs:
    canvas3D.getView().setWindowEyepointPolicy(View.RELATIVE_TO_SCREEN);    
    
    //Optional: Bei genderter "WindowEyepointPolicy": Verschiebung des Sichtbereichs 
    canvas3D.getView().setWindowMovementPolicy(View.VIRTUAL_WORLD);
    canvas3D.getView().setWindowResizePolicy(View.VIRTUAL_WORLD);
    
    
    //=====================================================
    //Transform fr die Box:
    Appearance appearanceGreen = new Appearance();
    ColoringAttributes coloringAttributesGreen = new ColoringAttributes();
    coloringAttributesGreen.setColor(new Color3f(Color.green));
    appearanceGreen.setColoringAttributes(coloringAttributesGreen);
    
    Box box = new Box (0.2f, 0.1f, 0.2f, appearanceGreen);

    //Rechtsverschiebung:
    Transform3D transform3dBox = new Transform3D();
    transform3dBox.setTranslation(new Vector3d (0.6f,0.0,0));
    
    // neue Transformgruppe
    TransformGroup transformGroupBox = new TransformGroup(transform3dBox);

    //Box an Transformgruppe hngen
    transformGroupBox.addChild(box);
    
    //=====================================================
    //Kegel!
    
    Appearance appearanceBlue = new Appearance();
    ColoringAttributes coloringAttributesBlue = new ColoringAttributes();
    coloringAttributesBlue.setColor(new Color3f(Color.blue));
    appearanceBlue.setColoringAttributes(coloringAttributesBlue);
    
    //Radius/Hhe:
    Cone cone = new Cone (0.2f, 0.8f, appearanceBlue);

    Transform3D transform3dCone = new Transform3D();
    transform3dCone.setTranslation(new Vector3d (-0.2f,0, 0));
    //Um 45 Grad an der x-Achse rotieren => Spitze geht auf User zu.
    transform3dCone.rotX( Math.PI / 4);
    
    // neue Transformgruppe
    TransformGroup transformGroupCone = new TransformGroup(transform3dCone);

    //Cone an Transformgruppe hngen
    transformGroupCone.addChild(cone);
    
    
    //================================================
    //Text in wei!
    
    Color3f colorWhite = new Color3f(Color.WHITE);
    Text2D text2D = new Text2D("Keks", colorWhite, "TimesNewRoman", 16, Font.BOLD);
    
    Transform3D transform3dText = new Transform3D();
    //Um 40% nach oben schieben:
    transform3dText.setTranslation(new Vector3d (0, 0.4f, 0));
    
    // neue Transformgruppe
    TransformGroup transformGroupText = new TransformGroup(transform3dText);

    //Text an Transformgruppe hngen
    transformGroupText.addChild(text2D);
    
    //================================================
    // Linie von "-1/-1/0" zu "+1/+1/0":
    Point3f[] points = new Point3f[2];
    points[0] = new Point3f(-1.0f, -1.0f, 0.0f);
    points[1] = new Point3f(1.0f, 1.0f, 0.0f);
    LineArray lineArray = new LineArray(2, LineArray.COORDINATES);
    lineArray.setCoordinates(0, points);
    Shape3D shapeLine = new Shape3D(lineArray, appearanceGreen);
    
    // neue Transformgruppe
    TransformGroup transformGroupLine = new TransformGroup();

    //Cone an Transformgruppe hngen
    transformGroupLine.addChild(shapeLine);
    
    //================================================
    //Alles zusammenbauen:
    
    //neue Branchgruppe
    BranchGroup branchgroup = new BranchGroup();

    //Transformgruppen an Branchgruppe hngen
    branchgroup.addChild(transformGroupBox);
    branchgroup.addChild(transformGroupCone);
    branchgroup.addChild(transformGroupText);
    branchgroup.addChild(transformGroupLine);
    
    // ... an das Universum hngen
    universe.addBranchGraph(branchgroup);
  }

  /**Main-Methode, zeigt das Java3D-Testfenster an.
   * 
   * @param args Kommandozeile, ungenutzt.
   */
  public static void main(String args[])
  {
    Java3dTestFrame java3dFrame = new Java3dTestFrame("Java3D-Tests");
    java3dFrame.setSize(600, 400);
    java3dFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    java3dFrame.setExtendedState(Frame.MAXIMIZED_BOTH);
    
    java3dFrame.setVisible(true);
  }
}
